/*
 *	pci.h
 *
 *	PCI defines and function prototypes
 *	Copyright 1994, Drew Eckhardt
 *	Copyright 1997--1999 Martin Mares <mj@ucw.cz>
 *
 *	For more information, please consult the following manuals (look at
 *	http://www.pcisig.com/ for how to get them):
 *
 *	PCI BIOS Specification
 *	PCI Local Bus Specification
 *	PCI to PCI Bridge Specification
 *	PCI System Design Guide
 */

#ifndef LINUX_PCI_H
#define LINUX_PCI_H


#include <linux/list.h>
#include <linux/module.h>
#include <linux/mod_devicetable.h>
#include <linux/device.h>
#include <linux/ioport.h>


/*
 * Under PCI, each device has 256 bytes of configuration address space,
 * of which the first 64 bytes are standardized as follows:
 */
#define PCI_VENDOR_ID				0x00	/* 16 bits */
#define PCI_DEVICE_ID				0x02	/* 16 bits */
#define PCI_COMMAND					0x04	/* 16 bits */
#define PCI_COMMAND_IO				0x1		/* Enable response in I/O space */
#define PCI_COMMAND_MEMORY			0x2		/* Enable response in Memory space */
#define PCI_COMMAND_MASTER			0x4		/* Enable bus mastering */
#define PCI_COMMAND_SPECIAL			0x8		/* Enable response to special cycles */
#define PCI_COMMAND_INVALIDATE		0x10	/* Use memory write and invalidate */
#define PCI_COMMAND_VGA_PALETTE		0x20	/* Enable palette snooping */
#define PCI_COMMAND_PARITY			0x40	/* Enable parity checking */
#define PCI_COMMAND_WAIT 			0x80	/* Enable address/data stepping */
#define PCI_COMMAND_SERR			0x100	/* Enable SERR */
#define PCI_COMMAND_FAST_BACK		0x200	/* Enable back-to-back writes */
#define PCI_COMMAND_INTX_DISABLE	0x400	/* INTx Emulation Disable */

#define PCI_STATUS					0x06	/* 16 bits */
#define PCI_STATUS_CAP_LIST			0x10	/* Support Capability List */
#define PCI_STATUS_66MHZ			0x20	/* Support 66 Mhz PCI 2.1 bus */
#define PCI_STATUS_UDF				0x40	/* Support User Definable Features [obsolete] */
#define PCI_STATUS_FAST_BACK		0x80	/* Accept fast-back to back */
#define PCI_STATUS_PARITY			0x100	/* Detected parity error */
#define PCI_STATUS_DEVSEL_MASK		0x600	/* DEVSEL timing */
#define PCI_STATUS_DEVSEL_FAST		0x000
#define PCI_STATUS_DEVSEL_MEDIUM	0x200
#define PCI_STATUS_DEVSEL_SLOW		0x400
#define PCI_STATUS_SIG_TARGET_ABORT	0x800	/* Set on target abort */
#define PCI_STATUS_REC_TARGET_ABORT	0x1000	/* Master ack of " */
#define PCI_STATUS_REC_MASTER_ABORT	0x2000	/* Set on master abort */
#define PCI_STATUS_SIG_SYSTEM_ERROR	0x4000	/* Set when we drive SERR */
#define PCI_STATUS_DETECTED_PARITY	0x8000	/* Set on parity error */

#define PCI_CLASS_REVISION		0x08	/* High 24 bits are class, low 8 revision */
#define PCI_REVISION_ID         0x08    /* Revision ID */
#define PCI_CLASS_PROG          0x09    /* Reg. Level Programming Interface */
#define PCI_CLASS_DEVICE        0x0a    /* Device class */

#define PCI_CACHE_LINE_SIZE		0x0c	/* 8 bits */
#define PCI_LATENCY_TIMER		0x0d	/* 8 bits */
#define PCI_HEADER_TYPE			0x0e	/* 8 bits */
#define PCI_HEADER_TYPE_NORMAL	0
#define PCI_HEADER_TYPE_BRIDGE	1
#define PCI_HEADER_TYPE_CARDBUS	2

#define PCI_BIST				0x0f	/* 8 bits */
#define PCI_BIST_CODE_MASK		0x0f	/* Return result */
#define PCI_BIST_START			0x40	/* 1 to start BIST, 2 secs or less */
#define PCI_BIST_CAPABLE		0x80	/* 1 if BIST capable */


struct pci_bus {
	struct list_head node;		/* node in list of buses */
	struct pci_bus	*parent;	/* parent bus this bridge is on */
	struct list_head children;	/* list of child buses */
	struct list_head devices;	/* list of devices on this bus */
	struct pci_dev	*self;		/* bridge device as seen by parent */
	struct resource	*resource[PCI_BUS_NUM_RESOURCES];
					/* address space routed to this bus */

	struct pci_ops	*ops;		/* configuration access functions */
	void		*sysdata;	/* hook for sys-specific extension */
	struct proc_dir_entry *procdir;	/* directory entry in /proc/bus/pci */

	unsigned char	number;		/* bus number */
	unsigned char	primary;	/* number of primary bridge */
	unsigned char	secondary;	/* number of secondary bridge */
	unsigned char	subordinate;	/* max number of subordinate buses */

	char		name[48];

	unsigned short  bridge_ctl;	/* manage NO_ISA/FBB/et al behaviors */
	unsigned short  pad2;
	struct device		*bridge;
	struct class_device	class_dev;
	struct bin_attribute	*legacy_io; /* legacy I/O for this bus */
	struct bin_attribute	*legacy_mem; /* legacy mem */
};

struct pci_dynids {
	spinlock_t lock;            /* protects list, index */
	struct list_head list;      /* for IDs added at runtime */
	unsigned int use_driver_data:1; /* pci_driver->driver_data is used */
};

/*
 * The pci_dev structure is used to describe PCI devices.
 */
struct pci_dev {
	struct list_head		global_list;	/* node in list of all PCI devices */
	struct list_head		bus_list;		/* node in per-bus list */
	struct pci_bus			*bus;			/* bus this device is on */
	struct pci_bus			*subordinate;	/* bus this device bridges to */

	void					*sysdata;		/* hook for sys-specific extension */
	struct proc_dir_entry	*procent;		/* device entry in /proc/bus/pci */

	unsigned int			devfn;			/* encoded device & function index */
	unsigned short			vendor;
	unsigned short			device;
	unsigned short			subsystem_vendor;
	unsigned short			subsystem_device;
	unsigned int			class;			/* 3 bytes: (base,sub,prog-if) */
	u8						hdr_type;		/* PCI header type (`multi' flag masked out) */
	u8						rom_base_reg;	/* which config register controls the ROM */
	struct pci_driver 		*driver;		/* which driver has allocated this device */
	u64						dma_mask;		/* Mask of the bits of bus address this
											   device implements.  Normally this is
											   0xffffffff.  You only need to change
											   this if your device has broken DMA
											   or supports 64-bit transfers.  */
	pci_power_t				current_state;  /* Current operating state. In ACPI-speak,
											   this is D0-D3, D0 being fully functional,
											   and D3 being off. */
	struct device			dev;			/* Generic device interface */

	/* device is compatible with these IDs */
	unsigned short 			vendor_compatible[DEVICE_COUNT_COMPATIBLE];
	unsigned short 			device_compatible[DEVICE_COUNT_COMPATIBLE];

	int						cfg_size;		/* Size of configuration space */

	/*
	 * Instead of touching interrupt line and base address registers
	 * directly, use the values stored here. They might be different!
	 */
	unsigned int			irq;
	struct resource 		resource[DEVICE_COUNT_RESOURCE]; /* I/O and memory regions + expansion ROMs */

	char 					*slot_name;		/* pointer to dev.bus_id */

	/* These fields are used by common fixups */
	unsigned int			transparent:1;	/* Transparent PCI bridge */
	unsigned int			multifunction:1;/* Part of multi-function device */
	/* keep track of device state */
	unsigned int			is_enabled:1;	/* pci_enable_device has been called */
	unsigned int			is_busmaster:1; /* device is busmaster */
	
	u32						saved_config_space[16]; /* config space saved at suspend time */
	struct bin_attribute	*rom_attr;				/* attribute descriptor for sysfs ROM entry */
	int						rom_attr_enabled;		/* has display of the rom attribute been enabled? */
	struct bin_attribute	*res_attr[DEVICE_COUNT_RESOURCE]; /* sysfs file for resources */
#ifdef CONFIG_PCI_NAMES
#define PCI_NAME_SIZE	96
#define PCI_NAME_HALF	__stringify(43)				/* less than half to handle slop */
	char					pretty_name[PCI_NAME_SIZE];	/* pretty name for users to see */
#endif
};

struct pci_driver {
	struct list_head node;
	char *name;
	struct module *owner;
	const struct pci_device_id *id_table;					/* must be non-NULL for probe to be called */
	int  (*probe)  (struct pci_dev *dev, const struct pci_device_id *id);	/* New device inserted */
	void (*remove) (struct pci_dev *dev);					/* Device removed (NULL if not a hot-plug capable driver) */
	int  (*suspend) (struct pci_dev *dev, pm_message_t state);	/* Device suspended */
	int  (*resume) (struct pci_dev *dev);	                /* Device woken up */
	int  (*enable_wake) (struct pci_dev *dev, u32 state, int enable);   /* Enable wake event */

	struct device_driver	driver;
	struct pci_dynids dynids;
};


extern int pci_module_init(struct pci_driver *drv);
extern int pci_unregister_driver(struct pci_driver *drv);
extern int pci_read_config_byte(struct pci_dev *dev, int where, u8 *val);
extern int pci_read_config_word(struct pci_dev *dev, int where, u16 *val);
extern int pci_write_config_word(struct pci_dev *dev, int where, u16 val);


#endif /* LINUX_PCI_H */
